#include <SoftwareSerial.h>
#include <MeetAndroid.h>

SoftwareSerial screen(7,6);
int led = 13;
boolean useScreen = true;

int buttonPress = A2;
int buttonQTC = A3;
int flexPointer = A4;
int flexThumb = A5;

MeetAndroid meetAndroid;

int initButPress;
int initQTC;

void setup()
{
  pinMode(led, OUTPUT);
  
  Serial.begin(57600);  // Begin the serial monitor at 9600bps

  if (useScreen) {
    screen.begin(9600);
    clearScreen();
    decreaseBrightness();
  }
  
  initButPress = analogRead(buttonPress);
  initQTC = analogRead(buttonQTC);
  
  // register the listening functions
  meetAndroid.registerFunction(name, 'p');
  meetAndroid.registerFunction(msg, 'm');
}

int stateQTC;
int statePress = LOW;
int lastStateQTC;
int lastStatePress;
long debounceDelay = 50;
long lastDebounceTime = 0;
void loop() {
  meetAndroid.receive(); // keep in loop() to receive events
  
  // do the following to keep track of possible button clicks
  // roughly follows http://arduino.cc/en/Tutorial/Debounce
  int readQTC = analogRead(buttonQTC);
  int readPress = analogRead(buttonPress);
  if (readQTC > 900) { readQTC = 255; }
  else { readQTC = 0; }
  if (readPress > 700) { readPress = 255; }
  else { readPress = 0; }
  
  // if the value is bouncing around threshold, reset timeout counter
  if (readQTC != lastStateQTC || readPress != lastStatePress)
    lastDebounceTime = millis();
    
  // if the timeout has triggered
  if ((millis() - lastDebounceTime) > debounceDelay) {
    
     // if the states are different, trigger a state change
    if (readQTC != stateQTC) {
      if (readQTC == 255)
        sendTextResponse();
      stateQTC = readQTC;
    } 
    
    if (readPress != statePress) {
      statePress = readPress;
    }
  }
  
  if (statePress == 255)
    changeBrightness();
  
  lastStateQTC = readQTC;  
}

void sendTextResponse() {
   meetAndroid.send("send response");
}

// flex sensor (forefinger [250,400])
void changeBrightness() {
  int val = analogRead(flexPointer);
  val = max(0, val - 250);
  val = (val * (29.0/150)) + 0.5;
  
  // flip the scale, and bump start to 128
  val = 29 - val;
  val += 128;
  
  // lcd brightness 128 - 157 (0-29)
  screen.write(0xFE);
  screen.write(val);  
}

char num[25];
char numTruncated[17];
void name(byte flag, byte numOfValues)
{
  meetAndroid.getString(num);
  selectLineOne();
  memcpy(numTruncated, num, 16);
  numTruncated[16] = '\0';
  screen.print(numTruncated);
}

char txt[141];
int strSize = 0;
void msg(byte flag, byte numOfValues)
{
  strSize = meetAndroid.getString(txt);
  scrollMessageSecondLine();
}
  
boolean doScroll = true;
void scrollMessageSecondLine() {
  if (doScroll) {
    selectLineTwo();
    if (strSize > 16) {
      // initial showing
      char cur[17];
      memcpy(cur, txt, 16);
      cur[16] = '\0';
      screen.print(cur);
      delay(1000);
      
      // scroll left
      for (int n = 0; n <= (strSize - 16); n++) {
        selectLineTwo();
        memcpy(cur, txt + n, 16);
        cur[16] = '\0';
        screen.print(cur);
        delay(333);
      }
      
      delay(1000);
      
      // scroll right
      for (int n = (strSize - 16); n >= 0; n--) {
        selectLineTwo();
        memcpy(cur, txt + n, 16);
        cur[16] = '\0';
        screen.print(cur);
        delay(333);
      }
      
      delay(1000);
    } else {
      screen.print(txt);
      delay(2500); 
    }
    
    clearScreen();
    memset(txt, '\0', 141);
  } else {
    clearScreen();
    selectLineTwo();
    screen.print(txt);
    delay(2500);
    clearScreen();
  }
}

void clearScreen()
{
  //clears the screen, you will use this a lot!
  screen.write(0xFE);
  screen.write(0x01); 
}
//-------------------------------------------------------------------------------------------
void selectLineOne()
{ 
  //puts the cursor at line 0 char 0.
  screen.write(0xFE); //command flag
  screen.write(128); //position
}
//-------------------------------------------------------------------------------------------
void selectLineTwo()
{ 
  //puts the cursor at line 0 char 0.
  screen.write(0xFE); //command flag
  screen.write(192); //position
}

void moveCursorRight(int numSpaces) {
  for (int n = 0; n < numSpaces; n++) {
    screen.write(0xFE);
    screen.write(20);
  }
}

void moveCursorLeft(int numSpaces) {
  for (int n = 0; n < numSpaces; n++) {
    screen.write(0xFE);
    screen.write(16); 
  }
}

void decreaseBrightness()
{
  screen.write(0x7C);
  screen.write(140);
}

void fullBrightness()
{
  screen.write(0x7C);
  screen.write(157); 
}
